#include "MyGLWidget.hpp"

#include "GLExtensions.hpp"


MyGLWidget::MyGLWidget(QWidget *parent)
    : QGLWidget(parent)
{
    startTimer(15); // shooting for 60 FPS
}

MyGLWidget::~MyGLWidget()
{
}

QSize MyGLWidget::minimumSizeHint() const
{
    return QSize(640, 640);
}

QSize MyGLWidget::sizeHint() const
{
    return QSize(800, 800);
}

void MyGLWidget::initializeGL()
{
    // Orthographic projection
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(-.5, .5, .5, -.5, -4, 15);

    // Enable smooth color gradients between vertices
    glShadeModel(GL_SMOOTH);

    // Identity modelview
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    // Background color is black
    glClearColor(0.0f, 0.0f, 0.0f, 0.0f);

    // Enable the depth test to cause faces to render from back to front
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LEQUAL);

    // Use the nicest perspective correction hint
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

    // Enable cull face so faces with normals pointing away from the camera
    // do not get rendered
    glEnable(GL_CULL_FACE);

    angle = 0.0f;
}

void MyGLWidget::paintGL()
{
    // Clear color (background) and depth (for depth test) buffers
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    // Rotate the modelview around the x axis
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glRotatef(angle, 1.0f, 0.0f, 0.0f);
    glRotatef(30.0f, 0.0f, 1.0f, 0.0f);

    angle += 1.0f; // degrees

    // Translate the multisampled version left
    glTranslatef(-0.3f, 0.0f, 0.0f);

    glEnable(GL_MULTISAMPLE);

    makeCube();

    glDisable(GL_MULTISAMPLE);

    // Translate the non-multisampled version right
    glTranslatef(0.6f, 0.0f, 0.0f);

    makeCube();
}

void MyGLWidget::makeCube()
{
    // Produce a cube by creating a quad strip to make a box with two open ends
    // and then fill in the ends with two quads (saves 6 glVertex3f calls)

    glBegin(GL_QUAD_STRIP);

        glColor3f(0.0f, 0.0f, 1.0f);
        glVertex3f(.2f, -.2f, .2f);
        glColor3f(0.0f, 1.0f, 0.0f);
        glVertex3f(-.2f, -.2f, .2f);

        glColor3f(1.0f, 0.0f, 0.0f);
        glVertex3f(.2f, .2f, .2f);
        glColor3f(0.0f, 1.0f, 1.0f);
        glVertex3f(-.2f, .2f, .2f);

        glColor3f(1.0f, 1.0f, 0.0f);
        glVertex3f(.2f, .2f, -.2f);
        glColor3f(1.0f, 0.0f, 1.0f);
        glVertex3f(-.2f, .2f, -.2f);

        glColor3f(0.0f, 0.0f, 1.0f);
        glVertex3f(.2f, -.2f, -.2f);
        glColor3f(0.0f, 1.0f, 0.0f);
        glVertex3f(-.2f, -.2f, -.2f);

        glColor3f(0.0f, 0.0f, 1.0f);
        glVertex3f(.2f, -.2f, .2f);
        glColor3f(0.0f, 1.0f, 0.0f);
        glVertex3f(-.2f, -.2f, .2f);

    glEnd();

    glBegin(GL_QUADS);

        glColor3f(1.0f, 0.0f, 1.0f);
        glVertex3f(.2f, -.2f, -.2f);
        glColor3f(0.0f, 1.0f, 1.0f);
        glVertex3f(.2f, -.2f, .2f);
        glColor3f(0.0f, 0.0f, 1.0f);
        glVertex3f(.2f, .2f, .2f);
        glColor3f(1.0f, 1.0f, 1.0f);
        glVertex3f(.2f, .2f, -.2f);

        glColor3f(1.0f, 0.0f, 1.0f);
        glVertex3f(-.2f, .2f, -.2f);
        glColor3f(0.0f, 1.0f, 1.0f);
        glVertex3f(-.2f, .2f, .2f);
        glColor3f(0.0f, 0.0f, 1.0f);
        glVertex3f(-.2f, -.2f, .2f);
        glColor3f(1.0f, 1.0f, 1.0f);
        glVertex3f(-.2f, -.2f, -.2f);

    glEnd();
}

void MyGLWidget::timerEvent(QTimerEvent *)
{
    // Redraw on a timer
    update();
}

void MyGLWidget::resizeGL(int width, int height)
{
    // Fix viewport to new size
    glViewport(0, 0, width, height);
}

void MyGLWidget::mousePressEvent(QMouseEvent *event)
{
}

void MyGLWidget::mouseMoveEvent(QMouseEvent *event)
{
}
